% EX_RSS_SURF  An example of ridge regression that constructs RSS
% (residual sum of squares) surfaces contour plots for different lambdas.
% 
% Usage:
%   ex_rss_surf; % Random data set will be generated and used
%   ex_rss_surf(csv_file_name); % Name of CSV file with input data
%
% See also RIDGE_SCALED
function ex_rss_surf(csvfilename)
close all;
if nargin == 0
    % Generate initial points and write them to CSV file    
    npoints = 250;
    x1 = 5*rand(npoints,1);
    x2 = x1 + 0.5*randn(npoints,1);
    y = 3 + 4*x1 + 5*x2 + 5*randn(npoints,1);
else
    dat = csvread(csvfilename);
    x1 = dat(:,1); x2 = dat(:,2); y = dat(:,3);
end
% Write initial points to CSV files
csvwrite('ridgedata.csv', [x1 x2 y]);
fd = fopen('ridgedata_comma.csv', 'w');
str = sprintf('%f %f %f\n', [x1 x2 y]');
fprintf(fd, '%s', strrep(str, '.', ','));
fclose(fd);
% Standardize (normalize) data
[yvst, xvst] = ridge_scale_data(y,[x1 x2]);
[b1m, b2m] = meshgrid(-1:0.01:1, -1:0.01:1);
ssm = nan(size(b1m));
% Draw the surfaces
for k = [0 0.1 1 5];
    % Calculate RSS(b1,b2) surface
    len = numel(ssm);
    b1v = reshape(b1m, len, 1);
    b2v = reshape(b2m, len, 1);    
    ssv = sum(([b1v b2v -ones(size(b2v))]*[xvst yvst]').^2, 2) + k*sum([b1v, b2v].^2, 2);
    ssm = reshape(ssv, size(ssm));
    % Show the surface as the countour plot
    figure;
    contour(b1m,b2m,log(ssm),'LineWidth', 2);
    [b,bnorm,sigma2, sigma2norm] = ridge_scaled(y, [x1 x2], k);
    fprintf('lambda = %f\n', k);
    fprintf('  bnorm = ['); fprintf('%7.3e ', bnorm); fprintf(']\n');
    fprintf('  b = ['); fprintf('%7.3e ', b); fprintf(']\n');    
    fprintf('  sigma2norm = %10.3e; sigmanorm = %10.3e\n', ...
        sigma2norm, sqrt(sigma2norm));    
    fprintf('  sigma2     = %10.3f; sigma     = %10.3f\n', ...
        sigma2, sqrt(sigma2));
    hold on;
    plot(bnorm(1),bnorm(2),'rd','LineWidth', 2);
    hold off;
    grid on;
    set(gca, 'FontSize',16);
    xlabel('\beta_1', 'FontSize', 18);
    ylabel('\beta_2', 'FontSize', 18);
end
% Show bias and variance of beta1
k = 0;
[b,bnorm,sigma2,sigma2norm,Cnorm] = ridge_scaled(y, [x1 x2], k);
sbnorm = sqrt(sigma2norm*diag(Cnorm));
sb = (sbnorm./bnorm).*b(2:end);
figure;
bv = (1:0.01:5.5);
plot(bv, normpdf(bv,b(2),sb(2)), 'r-', 'LineWidth', 2);
hold on;
k = 1;
[b,bnorm,sigma2,sigma2norm,Cnorm] = ridge_scaled(y, [x1 x2], k);
sbnorm = sqrt(sigma2norm*diag(Cnorm));
sb = (sbnorm./bnorm).*b(2:end);
plot(bv, normpdf(bv,b(2),sb(2)), 'b-', 'LineWidth', 2);
plot([4 4], [0 4], 'k-', 'LineWidth', 2);
hold off;
legend('\lambda=0', '\lambda=1', 'True value');
set(gca, 'FontSize', 16);
xlabel('\beta_2', 'FontSize', 18);
ylabel('{\it{p}}({\it{x}})', 'FontSize', 18);
end
