% RIDGE_SCALE_DATA  Data scaling (standardization)
% for the ridge regression procedure.
%
% Usage:
%   [yvst, xvst, coeff] = ridge_scale_data(yv,xv)
% Inputs:
%   yv -- Nx1 double -- dependent variable vector
%   xv -- NxK double -- independent variables matrix
%         (one variable -- one column)
% Outputs:
%   yvst -- scaled (standardized) yv variable
%   xvst -- scaled (standardized) xv variable
%   coeff -- struct with scaling coefficients
%
% Formula for scaling:
%   yst    = (y - mean(y)) ./ ynorm
%   xist   = (xi - mean(xi)) ./ xinorm
%   ynorm  = sqrt(sum( ((y - mean(y)).^2 ))
%   xinorm = sqrt(sum( ((xi - mean(xi)).^2 ))
% 
% N -- number of points
% K -- number of independent variables
% 
% See also RIDGE_SCALED
function [yvst, xvst, coeff] = ridge_scale_data(yv,xv)
% Check parameters size
if size(yv,1) ~= size(xv,1) || size(yv,2) ~= 1
   disp(size(yv));
   disp(size(xv));
   error('yv and xv sizes are not consistend');
end
% Scale data
ndims = size(xv, 2);
coeff = struct('xmean', nan(ndims,1), 'ymean', mean(yv),...
    'xnorm', nan(ndims,1), 'ynorm', sqrt(sum((yv - mean(yv)).^2)) ...
    );
xvst = nan(size(xv));
for i = 1:ndims
    coeff.xmean(i) = mean(xv(:,i));
    coeff.xnorm(i) = sqrt(sum((xv(:,i) - coeff.xmean(i)).^2));
    xvst(:,i) = (xv(:,i) - coeff.xmean(i)) ./ coeff.xnorm(i);
end
yvst = (yv - coeff.ymean)./coeff.ynorm;
end