% RIDGE_SCALED  Implementation of the ridge regression with X and Y
% variables scaling (standardization). It has some significant
% differences from MATLAB of GNU Octave ridge function:
%   * Another procedure of data scaling (standardization) that is
%     similar to used in StatGraphics
%   * Calculation of covariance matrix that contains VIFs for coefficients
%   * Calculation of the regression standard error
%
% Usage:
%   [b, bst, sigma2, sigma2st, Cnorm] = ridgeStandardized(y, x, k)
% Inputs:
%   y -- dependent variable vector
%   x -- independent variable matrix
%   k -- regularization parameter
% Outputs:
%   b -- ridge regression parameters
%     (y = b(1) + b(2)*x1 + ...)
%   bst -- ridge regression scaled (standardized) parameters
%     (y = bst(1)*x1 + bst(2)*x2 + ...)
%   sigma2 -- standard error of regression (not standardized data)
%   sigma2st -- standard error of regression (standardized data)
%   Cnorm -- covariance matrix divided by sigma^2.
%     Cnorm = W*(X'*X)*W; W = inv(X*X' * kI)
%     Its main diagonal elements can be considered as VIF (Variance
%     Inflation Factors) for the coefficients. Constant is not included!
%
% See also RIDGE_SCALE_DATA, RIDGE
function [b, bst, sigma2, sigma2st, Cnorm] = ridge_scaled(y, x, k)
% Check input data
y = y(:);
npoints = numel(y);
ncoeff = size(x,2);
if size(x,1) ~= npoints
    error('Invalid height of x matrix');
end
if ~isscalar(k) || ~isreal(k) || k < 0
    error('k must be a non-negative real scalar');
end
% Perform regression
[yvst, X, coeff] = ridge_scale_data(y, x);
W = inv(X'*X + k*eye(size(X,2)));
bst = W*X'*yvst;
b = descale_beta(bst, coeff);
Cnorm = W*(X'*X)*W;
% Calculate regression standard error
res = b(1) + sum(repmat(b(2:end),1,npoints)'.*x,2) - y;
resst = sum(repmat(bst,1,npoints)'.*X,2) - yvst;
sigma2 = (res'*res) ./ (npoints - ncoeff);
sigma2st = (resst'*resst) ./ (npoints - ncoeff);
end

function b = descale_beta(bnorm, coeff)
% Transform beta scaled (standardized) coefficients to non-scaled form
% Constant will be added to the beginning
    b = nan(numel(bnorm)+1,1);
    b(1) = coeff.ymean - coeff.ynorm.*sum(bnorm.*coeff.xmean./coeff.xnorm);
    b(2:end) = bnorm.*coeff.ynorm./coeff.xnorm;
end